<?php

namespace Drupal\k8s\Form;

use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Entity\EntityMalformedException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\k8s\Entity\K8sEntityBase;
use Drupal\k8s\Entity\K8sSchedule;
use Drupal\k8s\Service\K8sServiceException;

/**
 * Form controller for the Pod create form.
 *
 * @ingroup k8s
 */
class K8sPodCreateForm extends K8sCreateForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {
    $form = parent::buildForm($form, $form_state, $cloud_context);

    $form['time_scheduler'] = [
      '#type' => 'details',
      '#title' => $this->t('Time Scheduler'),
      '#open' => TRUE,
      '#weight' => $form['pod']['#weight'],
    ];

    $form['time_scheduler']['enable_time_scheduler'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable time scheduler'),
    ];

    $hour_options = [];
    for ($i = 0; $i < 24; $i++) {
      $hour_options[$i] = $i;
    }

    $minute_options = [];
    for ($i = 0; $i < 60; $i++) {
      $minute_options[$i] = $i;
    }

    $states = [
      'visible' => [
        ':input[name="enable_time_scheduler"]' => ['checked' => TRUE],
      ],
    ];

    $form['time_scheduler']['container'] = [
      '#type' => 'container',
      '#states' => $states,
    ];

    $form['time_scheduler']['container']['start_time_title'] = [
      '#type' => 'item',
      '#title' => 'Start-up Time',
    ];

    $hour_options = [];
    for ($i = 0; $i < 24; $i++) {
      $hour_options[$i] = $i;
    }

    $minute_options = [];
    for ($i = 0; $i < 60; $i++) {
      $minute_options[$i] = $i;
    }

    $form['time_scheduler']['container']['start_hour'] = [
      '#type' => 'select',
      '#options' => $hour_options,
      '#default_value' => 0,
      '#prefix' => '<div class= "container-inline">',
    ];

    $form['time_scheduler']['container']['start_minute'] = [
      '#type' => 'select',
      '#options' => $minute_options,
      '#default_value' => 0,
      '#prefix' => ': ',
      '#suffix' => '</div>',
    ];

    $form['time_scheduler']['container']['stop_time_title'] = [
      '#type' => 'item',
      '#title' => 'Stop Time',
    ];

    $form['time_scheduler']['container']['stop_hour'] = [
      '#type' => 'select',
      '#options' => $hour_options,
      '#default_value' => 6,
      '#prefix' => '<div class= "container-inline">',
    ];

    $form['time_scheduler']['container']['stop_minute'] = [
      '#type' => 'select',
      '#options' => $minute_options,
      '#default_value' => 0,
      '#prefix' => ': ',
      '#suffix' => '</div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *    Thrown when unable to create entity.
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    $this->k8sService->setCloudContext($cloud_context);

    $params = Yaml::decode($entity->getDetail());

    if (!empty($entity->getOwner())) {
      // Add owner uid to annotations.
      $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_CREATED_BY_UID] = $entity->getOwner()->id();
    }

    try {
      $result = $this->k8sService->createPod($entity->getNamespace(), $params);

      $entity->setName($result['metadata']['name']);
      if (method_exists($entity, 'setCreationYaml')) {
        $entity->setCreationYaml($entity->getDetail());
      }
      $entity->save();

      // Update the entity.
      $this->k8sService->updatePods([
        'metadata.name' => $entity->getName(),
      ], FALSE);

      $this->processOperationStatus($entity, 'created');

      if (!empty($form_state->getValue('enable_time_scheduler'))) {
        $timestamp = time();
        $schedule = K8sSchedule::create([
          'cloud_context' => $cloud_context,
          'name' => $entity->getNamespace() . '_' . $entity->getName(),
          'namespace_name' => $entity->getNamespace(),
          'pod_name' => $entity->getName(),
          'start_hour' => $form_state->getValue('start_hour'),
          'start_minute' => $form_state->getValue('start_minute'),
          'stop_hour' => $form_state->getValue('stop_hour'),
          'stop_minute' => $form_state->getValue('stop_minute'),
          'manifest' => $entity->getCreationYaml(),
          'created' => $timestamp,
          'changed' => $timestamp,
          'refreshed' => $timestamp,
        ]);
        $schedule->save();
      }

      $form_state->setRedirect("view.k8s_pod.list", ['cloud_context' => $entity->getCloudContext()]);
    }
    catch (K8sServiceException
    | EntityStorageException
    | EntityMalformedException $e) {

      try {
        $this->processOperationErrorStatus($entity, 'created');
      }
      catch (EntityMalformedException $e) {
        $this->handleException($e);
      }
    }
  }

}
