<?php

namespace Drupal\k8s\Plugin\cloud\store;

use Drupal\cloud\Entity\CloudStoreInterface;
use Drupal\cloud\Plugin\cloud\CloudPluginBase;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Plugin\cloud\store\CloudStorePluginInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\k8s\Controller\K8sCostsControllerBase;
use Drupal\k8s\Service\K8sServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * K8s Cloud Store plugin.
 */
class K8sCloudStorePlugin extends CloudPluginBase implements CloudStorePluginInterface, ContainerFactoryPluginInterface {

  /**
   * Variable to compare with the time difference.
   *
   * @var int
   */
  private $length = 60 * 60;

  /**
   * The K8s Service.
   *
   * @var \Drupal\k8s\Service\K8sServiceInterface
   */
  protected $k8sService;

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * Current login user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * Entity link renderer object.
   *
   * @var \Drupal\cloud\Service\EntityLinkRendererInterface
   */
  protected $entityLinkRenderer;

  /**
   * File system object.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The class resolver.
   *
   * @var \Drupal\Core\DependencyInjection\ClassResolverInterface
   */
  protected $classResolver;

  /**
   * K8sCloudStorePlugin constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\k8s\Service\K8sServiceInterface $k8s_service
   *   The Kubernetes cluster.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Component\Uuid\UuidInterface $uuid_service
   *   The uuid service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   Current login user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   *   The class resolver.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              K8sServiceInterface $k8s_service,
                              EntityTypeManagerInterface $entity_type_manager,
                              UuidInterface $uuid_service,
                              AccountProxyInterface $current_user,
                              ConfigFactoryInterface $config_factory,
                              CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
                              EntityLinkRendererInterface $entity_link_renderer,
                              FileSystemInterface $file_system,
                              ClassResolverInterface $class_resolver) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->k8sService = $k8s_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->uuidService = $uuid_service;
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->entityLinkRenderer = $entity_link_renderer;
    $this->fileSystem = $file_system;
    $this->classResolver = $class_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('k8s'),
      $container->get('entity_type.manager'),
      $container->get('uuid'),
      $container->get('current_user'),
      $container->get('config.factory'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('entity.link_renderer'),
      $container->get('file_system'),
      $container->get('class_resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityBundleName(): string {
    return $this->pluginDefinition['entity_bundle'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildListHeader(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildListRow(CloudStoreInterface $entity): array {
    return [];
  }

  /**
   * Load all entities for a given entity type and bundle.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   Array of Entity Interface.
   */
  public function loadConfigEntities(): EntityInterface {
    return $this->entityTypeManager->getStorage($this->pluginDefinition['entity_type'])
      ->loadByProperties(['type' => [$this->pluginDefinition['entity_bundle']]]);
  }

  /**
   * Load a cloud service provider (CloudConfig) entity.
   *
   * @param string $entity_bundle
   *   Bundle string.
   *
   * @return array
   *   Entity or empty if there is no entity.
   */
  public function loadConfigEntity($entity_bundle): array {
    $entity = $this->entityTypeManager
      ->getStorage($this->pluginDefinition['entity_type'])
      ->loadByProperties(['entity_bundle' => [$entity_bundle]]);

    if (count($entity) === 1) {
      return array_shift($entity);
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function updateK8sResourceStoreEntity(): void {
    // Set queue.
    $queue = \Drupal::queue('k8s_update_resources_queue');
    $entities = \Drupal::service('plugin.manager.cloud_config_plugin')->loadConfigEntities('k8s');
    if (empty($entities)) {
      return;
    }

    // Get target entities to be updated.
    foreach ($entities ?: [] as $k8s_cluster) {
      $cloud_context = $k8s_cluster->getCloudContext();

      // To check whether k8s metrics server is installed or not.
      if (!k8s_is_metrics_enabled($cloud_context)) {
        $this->messenger->addWarning($this->t('@message Please install @metrics_server_link into @cloud_context', [
          '@message' => $this->t('Node Resource Store requires K8s Metrics Server.'),
          '@metrics_server_link' => $this->k8sService->getMetricsServerLink($cloud_context),
          '@cloud_context' => $cloud_context,
        ]));
        continue;
      }

      $namespace_entities = $this->entityTypeManager
        ->getStorage('k8s_namespace')
        ->loadByProperties([
          'cloud_context' => $cloud_context,
        ]);
      if (empty($namespace_entities)) {
        continue;
      }

      foreach ($namespace_entities ?: [] as $namespace) {
        $controller = $this->classResolver->getInstanceFromDefinition(K8sCostsControllerBase::class);
        $cost_types = array_keys($controller->getEc2CostTypes());
        foreach ($cost_types ?: [] as $cost_type) {
          // Get resource metrics from each namespace.
          $queue->createItem([
            'k8s_method_name' => 'updateK8sNamespaceResourceStoreEntity',
            'cloud_context' => $cloud_context,
            'namespace' => $namespace->getName(),
            'cost_type' => $cost_type,
          ]);
        }
      }
    }
  }

}
